<?php

declare(strict_types=1);

namespace mirzaev\huesos\models\acquirings;

// Files of the project
use mirzaev\huesos\models\core,
	mirzaev\huesos\models\settings,
	mirzaev\huesos\models\cart,
	mirzaev\huesos\models\order,
	mirzaev\huesos\models\enumerations\currency;

// Framework for ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Library for ArangoDB
use ArangoDBClient\Document as _document;

// Built-in libraries
use exception,
	DomainException as exception_domain,
	RuntimeException as exception_runtime;

/**
 * Model of robokassa
 *
 * @package mirzaev\huesos\models\acquirings
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class robokassa extends core
{
	/**
	 * Link
	 *
	 * Generate the link to pay for the order
	 *
	 * @param order $order The order
	 * @param array &$errors Registry of errors
	 *
	 * @return string|null The link to pay for the order, if generated
	 */
	public static function link(order $order, array &$errors = []): ?string
	{
		try {
			// Initializing the settings
			$settings = settings::active();

			if ($settings instanceof settings) {
				// Initialized the settings

				if ($settings->acquirings['robokassa']['enabled']) {
					// Enabled the robokassa acquiring

					// Declaring the robokassa settings
					$robokassa = null;

					// Initializing the test mode register
					$test = 0;

					if ($settings->acquirings['robokassa']['mode'] === 'work') {
						// Work mode

						// Initializing the robokassa settings
						$robokassa = require(SETTINGS . DIRECTORY_SEPARATOR . 'acquirings' . DIRECTORY_SEPARATOR . 'robokassa' . DIRECTORY_SEPARATOR . 'work.php');
					} else if ($settings->acquirings['robokassa']['mode'] === 'test') {
						// Test mode

						// Reinitializing the test mode register
						$test = 1;

						// Initializing the robokassa settings
						$robokassa = require(SETTINGS . DIRECTORY_SEPARATOR . 'acquirings' . DIRECTORY_SEPARATOR . 'robokassa' . DIRECTORY_SEPARATOR . 'test.php');
					} else {
						// Failed to initialized the mode

						// Exit (fail)
						throw new exception_domain('Failed to initialize the robokassa mode');
					}

					// Initializing the cart
					$cart = $order->cart();

					if ($cart instanceof cart) {
						// Initialized the cart

						// Initializing robokassa parameters
						$login = $robokassa['login'];
						$password = $robokassa['passwords'][0];

						// Initializing the order parameters
						$identifier = $order->getKey();

						// Initializing the invoice parameters
						$description = "Оплата заказа";
						$cost = $cart->cost() + ($cart->buffer['delivery']['cost'] ?? 0);

						// Generatings the MD5 hash
						$md5 = md5("$login:$cost:$identifier:$password");
/*
						// Writing the MD5 hash into the order implementator
						$order->acquirings = ['robokassa' => ['hash' => $md5] + ($order->acquirings['robokassa'] ?? [])] + ($order->acquirings ?? []);

						if (document::update($order->__document(), errors: $errors)) {
							// Writed to ArangoDB */

							// Exit (success)
							return "https://auth.robokassa.ru/Merchant/Index.aspx?MerchantLogin=$login&OutSum=$cost&InvId=$identifier&Description=$description&SignatureValue=$md5&IsTest=$test";
						/* } else throw new exception('Failed to write the robokassa hash into the order'); */
					} else {
						// Not initialized the cart

						// Exit (fail)
						throw new exception_runtime('Failed to initialize the cart');
					}
				}
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Verify (result)
	 *
	 * Generate hash by arguments and verify for result with the hash
	 *
	 * @param int $identifier Identifier of the order
	 * @param string $cost Cost of the order (can be with two zeros, like 1000.00)
	 * @param string $hash Hash of the order
	 * @param array &$errors Registry of errors
	 *
	 * @return string|null The link to pay for the order, if generated
	 */
	public static function result(int $identifier, string $cost,	string $hash, bool $test = false, array &$errors = []): ?bool
	{
		try {
			// Initializing the settings
			$settings = settings::active();

			if ($settings instanceof settings) {
				// Initialized the settings

				if ($settings->acquirings['robokassa']['enabled']) {
					// Enabled the robokassa acquiring

					// Declaring the robokassa settings
					$robokassa = null;

					if ($test) {
						// Test mode

						// Initializing the robokassa settings
						$robokassa = require(SETTINGS . DIRECTORY_SEPARATOR . 'acquirings' . DIRECTORY_SEPARATOR . 'robokassa' . DIRECTORY_SEPARATOR . 'test.php');
					} else {
						// Work mode

						// Initializing the robokassa settings
						$robokassa = require(SETTINGS . DIRECTORY_SEPARATOR . 'acquirings' . DIRECTORY_SEPARATOR . 'robokassa' . DIRECTORY_SEPARATOR . 'work.php');
					}

					// Initializing robokassa parameters
					$password = $robokassa['passwords'][1];

					// Generatings the MD5 hash
					$md5 = strtoupper(md5("$cost:$identifier:$password"));

					// Exit (success)
					return $md5 === $hash;
				}
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Verify (success)
	 *
	 * Generate hash by arguments and verify for success with the hash
	 *
	 * @param int $identifier Identifier of the order
	 * @param string $cost Cost of the order (can be with two zeros, like 1000.00)
	 * @param string $hash Hash of the order
	 * @param array &$errors Registry of errors
	 *
	 * @return string|null The link to pay for the order, if generated
	 */
	public static function success(int $identifier, string $cost,	string $hash, bool $test = false, array &$errors = []): ?bool
	{
		try {
			// Initializing the settings
			$settings = settings::active();

			if ($settings instanceof settings) {
				// Initialized the settings

				if ($settings->acquirings['robokassa']['enabled']) {
					// Enabled the robokassa acquiring

					// Declaring the robokassa settings
					$robokassa = null;

					if ($test) {
						// Test mode

						// Initializing the robokassa settings
						$robokassa = require(SETTINGS . DIRECTORY_SEPARATOR . 'acquirings' . DIRECTORY_SEPARATOR . 'robokassa' . DIRECTORY_SEPARATOR . 'test.php');
					} else {
						// Work mode

						// Initializing the robokassa settings
						$robokassa = require(SETTINGS . DIRECTORY_SEPARATOR . 'acquirings' . DIRECTORY_SEPARATOR . 'robokassa' . DIRECTORY_SEPARATOR . 'work.php');
					}

					// Initializing robokassa parameters
					$password = $robokassa['passwords'][0];

					// Generatings the MD5 hash
					$md5 = md5("$cost:$identifier:$password");

					// Exit (success)
					return $md5 === $hash;
				}
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}
}
