<?php

declare(strict_types=1);

namespace mirzaev\huesos\models;

// Files of the project
use mirzaev\huesos\models\core,
	mirzaev\huesos\models\cart,
	mirzaev\huesos\models\account,
	mirzaev\huesos\models\reservation,
	mirzaev\huesos\models\traits\document as document_trait,
	mirzaev\huesos\models\interfaces\document as document_interface,
	mirzaev\huesos\models\interfaces\collection as collection_interface,
	mirzaev\huesos\models\enumerations\language,
	mirzaev\huesos\models\enumerations\currency;

// Framework for ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Library for ArangoDB
use ArangoDBClient\Document as _document;

// Built-in libraries
use exception;

/**
 * Model of order
 *
 * @uses !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * @package mirzaev\huesos\models
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class order extends core implements document_interface, collection_interface
{
	use document_trait;

	/**
	 * Name of the collection in ArangoDB
	 */
	final public const string COLLECTION = 'order';

	/**
	 * Cart
	 *
	 * Search for the connected cart
	 *
	 * @param array &$errors Registry of errors
	 *
	 * @return cart|null The connected cart, if found
	 */
	public function cart(array &$errors = []): ?cart
	{
		try {
			if ($result = collection::execute(
				<<<'AQL'
				FOR v IN 1..1 INBOUND @order GRAPH users
					FILTER PARSE_IDENTIFIER(v._id).collection == @cart
					LIMIT 1
					return v
				AQL,
				[
					'order' => $this->document->getId(),
					'cart' => cart::COLLECTION
				],
				errors: $errors
			)) {
				// Found the instance of the cart connected to the order

				// Initializing the object
				$cart = new cart;

				if (method_exists($cart, '__document')) {
					// Object can implement a document from ArangoDB

					// Writing the instance of the cart document from ArangoDB to the implement object
					$cart->__document($result);

					// Exit (success)
					return $cart;
				}
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}
		// Exit (fail)
		return null;
	}

	/**
	 * Account
	 *
	 * Search for the connected account
	 *
	 * @param array &$errors Registry of errors
	 *
	 * @return account|null The connected account, if found
	 */
	public function account(array &$errors = []): ?account
	{
		try {
			if ($result = collection::execute(
				<<<'AQL'
				FOR v IN 1..1 OUTBOUND @order GRAPH users
					FILTER PARSE_IDENTIFIER(v._id).collection == @account
					LIMIT 1
					return v
				AQL,
				[
					'order' => $this->document->getId(),
					'account' => account::COLLECTION
				],
				errors: $errors
			)) {
				// Found the instance of the account connected to the account

				// Initializing the object
				$account = new account;

				if (method_exists($account, '__document')) {
					// Object can implement a document from ArangoDB

					// Implementinf parameters
					if (isset($result->language)) $result->language = language::{$result->language};
					if (isset($result->currency)) $result->currency = currency::{$result->currency};

					// Writing the instance of the account document from ArangoDB to the implement object
					$account->__document($result);

					// Exit (success)
					return $account;
				}
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}
		// Exit (fail)
		return null;
	}

}
